#include "ArrayP2d.hpp"
// Constructor
ArrayP2d::ArrayP2d(unsigned int sizeVal, int i_deltaVal, char* str_class_nameVal) {
	assert(str_class_nameVal);      // null pointer to class name
	assert(str_class_nameVal[0]);   // name not valid
	str_class_name = str_class_nameVal;

//	pt = (Point2d*)malloc(sizeof(Point2d)*sizeVal);
	pt = new Point2d[sizeVal];
	assert(pt);
	u_start_array_size = sizeVal;
	allocated_objects = sizeVal;
	i_delta = i_deltaVal;
	Clear();
}

// Destructor
ArrayP2d::~ArrayP2d(void) {
	#ifdef DEBUG
	assert(pt!=MK_FP(0xCCCC, 0xCCCC));
	#endif

//	if(pt) free(pt);
	if(pt) delete [] pt;
}

int ArrayP2d::IsInArray(Point2d& p) {

	for(int i=0; i<n_objects; i++) {
		if(pt[i] == p) return i;
	}
	return -1;
}

void ArrayP2d::Print(void) {
	for(int i=0; i<n_objects; i++) {
		pt[i].Print();
	}
}



// operator []
Point2d& ArrayP2d::operator[](unsigned int i){

	#ifdef DEBUG
	#define operator_brackets__OUT_OF_ALLOCATED_SPACE   \
		i<allocated_objects //&& i>=0
	assert(operator_brackets__OUT_OF_ALLOCATED_SPACE);
	#define operator_brackets__UNDEFINED_OBJECT        i<n_objects
	assert(operator_brackets__UNDEFINED_OBJECT);
	#endif

	return pt[i];

}


// operator =
  ArrayP2d& ArrayP2d::operator = (ArrayP2d& at){

//	assert(0);   // CAN'Point2d COPY HARD DATA

	#ifdef DEBUG
	assert(!strcmp(str_class_name, at.str_class_name));
	assert(at.pt);
	assert(at.allocated_objects);
	#endif

	Clear();
	i_delta = at.i_delta;
	u_start_array_size = at.u_start_array_size;

	for(unsigned int i=0; i<at.Count(); i++){
		AddNext(at.pt[i]);
	}

	//allocated_objects = t.allocated_objects;
	return *this;
}


// AddNext(Point2d& t)
void ArrayP2d::AddNext(Point2d& t) {

	if(n_objects >= allocated_objects) {
		SetSize((long)allocated_objects+(long)i_delta);
	}

	#ifdef DEBUG
	#define AddNext__STATIC_SIZED_ARRAY_IS_FULL   n_objects < allocated_objects
	assert(AddNext__STATIC_SIZED_ARRAY_IS_FULL);
	#endif


	pt[n_objects] = t;
	n_objects++;

}


// ReplaceAt
  void ArrayP2d::ReplaceAt(Point2d& t, unsigned int i_at) {
	#ifdef DEBUG
	#define Replace__OUT_OF_DEFINED_OBJECTS   i_at<n_objects //&& i_at>=0
	assert(Replace__OUT_OF_DEFINED_OBJECTS);
	#endif

	pt[i_at] = t;

}

// InsertAt(Point2d& t)
  void ArrayP2d::InsertAt(Point2d& t, unsigned int i_at) {

	if(n_objects >= allocated_objects) {
		SetSize((long)allocated_objects+(long)i_delta);
	}

	#ifdef DEBUG
	#define InsertAt__OUT_OF_DEFINED_OBJECTS   i_at<=n_objects //&& i_at>=0
	assert(InsertAt__OUT_OF_DEFINED_OBJECTS);
	#define InsertAt__ARRAY_FULL     n_objects != allocated_objects
	assert(InsertAt__ARRAY_FULL);
	#endif
		 // dest, src
//	assert(0);
	memmove(&pt[i_at+1], &pt[i_at], sizeof(Point2d)*(n_objects-i_at));

	pt[i_at] = t;
	n_objects++;

}



// DeleteAt(unsigned int i_at)
  void ArrayP2d::DeleteAt(unsigned int i_at) {

//	assert(0);  // VOY A INTENTAR MOVER AL FINAL DE LA LISTA EL
				// ELEMENTO QUE QUIERO BORRAR.
				// ASI NO TENGO QUE CREAR NI DESTRUIR OBJETOS

	#ifdef DEBUG
	#define Delete__OUT_OF_DEFINED_OBJECTS    i_at<n_objects //&& i_at>=0
	assert(Delete__OUT_OF_DEFINED_OBJECTS);
	#define Delete__ARRAY_EMPTY     n_objects>0
	assert(Delete__ARRAY_EMPTY);
	#endif

//	assert(0);

	unsigned char * buffer = new unsigned char[sizeof(Point2d)];
	assert(buffer);
	// save pt[i_at] in a temporal buffer
	memmove(buffer, &pt[i_at], sizeof(Point2d));//dest,src

	// shift left  objects in list
	memmove(&pt[i_at], &pt[i_at+1], sizeof(Point2d)*(n_objects-i_at-1));//dest,src

	// restore pt[i_at] from buffer, and places it at the end of the ArrayP2d
	memmove(&pt[n_objects-1], buffer, sizeof(Point2d));//dest,src

	delete buffer;



	n_objects--;

}

// Clear(void)
  void ArrayP2d::Clear(void) {
	n_objects=0;
	SetSize(u_start_array_size);
}

void ArrayP2d::SetSize(long new_allocated_objects) {
//	assert(0);

	Point2d* p_new_block;
	//long new_allocated_objects;
	long new_size_bytes;

	new_size_bytes = sizeof(Point2d)*new_allocated_objects;

	#ifdef DEBUG
	#define SetSize_ARRAY_SIZE_GREATER_THAN_64Kb   NULL
	if(new_size_bytes > 65500l) assert(SetSize_ARRAY_SIZE_GREATER_THAN_64Kb);
	#endif

	// p_new_block = (Point2d*)realloc(pt,new_size_bytes); // realloc works well

	p_new_block = new Point2d[new_allocated_objects];
//	memcpy(&p_new_block[0], &pt[0], allocated_objects*sizeof(Point2d));
	unsigned int common_objects = MIN(new_allocated_objects,allocated_objects);
	for(unsigned int i=0; i<common_objects; i++) {
		p_new_block[i] = pt[i];
	}
	delete [] pt;

/*
	p_new_block = (Point2d*)malloc(new_size_bytes);
	memcpy(p_new_block, pt, allocated_objects*sizeof(Point2d));
	free(pt);
*/
	if(p_new_block) {
		pt = p_new_block;
		allocated_objects = new_allocated_objects;
	}
	else {
		#ifdef DEBUG
		#define SetSize__REALLOCATE_ERROR   NULL
		assert(SetSize__REALLOCATE_ERROR);
		#endif
	}

//	printf("\nn_objects = %i     alloc_objects = %i\n",
//		n_objects,
//		allocated_objects
//	);


}

