#include "Segment.hpp"
#include <assert.h>
#include <values.h>
#include <math.h>
#include <stdio.h>

#define TYPE_Y 1
#define TYPE_X 2

Segment Segment::operator = (Segment& s) {
	p1 = s.p1;
	p2 = s.p2;
	return (* this);
}

float Segment::Distance(Vector& point) {
	return sqrt(Distance2(point));
}

float Segment::Distance2(Vector& point) {
/*
	Segment s_perp;
	Vector v_dir, v_dir_perp;
	Vector v_intersection;
	Vector v_dist;

	v_dir = p2-p1;

	v_dir_perp.x =  v_dir.y;
	v_dir_perp.y = -v_dir.x;

	s_perp.p1 = point - v_dir_perp;
	s_perp.p2 = point + v_dir_perp;

//	setcolor(DARKGRAY);
//	s_perp.Draw();

	v_intersection = Intersect(s_perp);
	v_dist = point - v_intersection;
*/
	Vector v_intersection;
	Vector v_dist;

	v_intersection = GetNearestPoint(point);
	v_dist = point - v_intersection;

	float f_dist;
	f_dist = (v_dist|v_dist);
	return f_dist;

}

Vector Segment::GetNearestPoint(Vector& point) {

	Segment s_perp;
	Vector v_dir, v_dir_perp;
	Vector v_intersection(1,2,3);
	Vector v_dist;

	v_dir = p2-p1;

	v_dir_perp.x =  v_dir.y;
	v_dir_perp.y = -v_dir.x;

	s_perp.p1 = point - v_dir_perp;
	s_perp.p2 = point + v_dir_perp;

	Intersect(s_perp, &v_intersection);

	return v_intersection;
}


int Segment::Intersect(Segment& s1, Vector* v_result) {

	double m1, c1;
	double m2, c2;
	int type1, type2;
	int is_ok;

	// if segment length is 0!
	if(s1.p1==s1.p2 || p1==p2) return NULL;

	*v_result = Vector(0,0,0);

	Get_m_c_type(s1, m1, c1, type1);
	Get_m_c_type(*this, m2, c2, type2);

	if(type1 == TYPE_Y && type2 == TYPE_Y) {
		Vector v;
		is_ok = Intersect_type_YY(m1,c1,m2,c2, &v);
		v_result->x = v.y;
		v_result->y = v.x;
		return is_ok;
	}

	if(type1 == TYPE_X && type2 == TYPE_X)
		return Intersect_type_YY(m1,c1,m2,c2, v_result);

	if(type1 == TYPE_Y && type2 == TYPE_X) {
		Vector v;
		is_ok = Intersect_type_YX(m1,c1,m2,c2, &v);
		v_result->x = v.y;
		v_result->y = v.x;
		return is_ok;
	}

	if(type1 == TYPE_X && type2 == TYPE_Y)
		return Intersect_type_YX(m1,c1,m2,c2, v_result);

	assert(0);
	return NULL;
}

// eq type A:    y = m1*x + c1   y = m2*x + c2
int Segment::Intersect_type_YY(double m1, double c1, double m2, double c2, Vector* v_result) {
//	Vector v;
	float denom;
	denom = m1-m2;
	if( denom == 0.0) return NULL;
	v_result->x = (c2-c1) / denom;
	v_result->y = m1 * v_result->x + c1;
	return 1;
}

// eq type B:    y = m1*x + c1   x = m2*y + c2
int Segment::Intersect_type_YX(double m1, double c1, double m2, double c2, Vector* v_result) {
//	Vector v;
	float denom;
	denom = 1.0-m1*m2;
	if( denom == 0.0) return NULL;
	v_result->y = (m1*c2+c1) / denom;
	v_result->x = m2 * v_result->y + c2;
	return 1;
}

void Segment::Get_m_c_type(Segment& seg, double& m, double& c, int& type) {

	float seg_p2y_p1y;
	float seg_p2x_p1x;

	seg_p2y_p1y = seg.p2.y - seg.p1.y;
	seg_p2x_p1x = seg.p2.x - seg.p1.x;

	if(ABS(seg_p2x_p1x) <= ABS(seg_p2y_p1y)) {
		type = TYPE_Y;
		if(seg_p2y_p1y==(float)0.0)   {
			type = TYPE_Y;
			type = TYPE_Y;
			type = TYPE_Y;
		}
		assert(seg_p2y_p1y); // the segment length is 0 !
		m = seg_p2x_p1x / seg_p2y_p1y;
		c = seg.p1.x - m * seg.p1.y;
	}
	else {
		type = TYPE_X;
		assert(seg_p2x_p1x); // the segment length is 0 !
		m = seg_p2y_p1y / seg_p2x_p1x;
		c = seg.p1.y - m * seg.p1.x;
	}
}




int Segment::AreInSameSide(Point2d& pa, Point2d& pb) {
	Vector v_dir;
	Vector v1(pa.x, pa.y,0);
	Vector v2(pb.x, pb.y,0);

	v_dir = p2-p1;
	float aux = v_dir.x;
	v_dir.x = -v_dir.y;
	v_dir.y = aux;

	v1 = v1-p1;
	v2 = v2-p1;

	float sign1 = v_dir|v1;
	float sign2 = v_dir|v2;

	if((sign1>0 && sign2>0) || (sign1<0 && sign2<0)) return 1;
	return 0;
}


int Segment::IsInSegment(Vector& p) {
#define MIN(A,B) (((A)<(B)) ? (A) : (B))
#define MAX(A,B) (((A)>(B)) ? (A) : (B))

	float x_min = MIN(p1.x, p2.x);
	float x_max = MAX(p1.x, p2.x);

	float y_min = MIN(p1.y, p2.y);
	float y_max = MAX(p1.y, p2.y);

	#ifdef DRAW_DEBUG
	printf("IsInSegment ");
	Print();
	printf("\n");
	Print();
	#endif
//	printf("\n");

	#define EPSILON1 0.0001
	if(x_max-x_min < EPSILON1) {
		x_min -= EPSILON1;
		x_max += EPSILON1;
	}

	if(y_max-y_min < EPSILON1) {
		y_min -= EPSILON1;
		y_max += EPSILON1;
	}

	if( p.x >= x_min && p.x <= x_max &&
		p.y >= y_min && p.y <= y_max) {
		#ifdef DRAW_DEBUG
		setcolor(BROWN);
		Draw();
		printf("  YES\n");
		getch();
		#endif
		return 1;
	}

	#ifdef DRAW_DEBUG
	setcolor(BROWN);
	Draw();
	printf("  NO\n");
	getch();
	#endif

	return 0;

}
